unit Utimein;

{
TTimeIn class reads timecodes from Adrienne time code reader cards.

*** It is implemented as a separate thread ***

To use this class:

	1. create an instance like this:
				var
					TimeIn:TTimeIn;
				...
				TimeIn := TTimeIn.Create(baseaddress, nil)

		 where baseaddress is $CC00 or whatever you changed the jumper to
		 (If you want to use ClockBox to show the frames, pass ClockBox instead of nil.)

	2. Set the callback like this:

				TimeIn.OnNewFrame := someprocedure;

		 and define someprocedure as what you want to do on each new frame.


There is also a function to add frames with the drop-frame calculation - see AddFrames.

Written by In-key Information Group - www.inkey.com - Feb 1999
}

interface

uses SysUtils, Windows, Classes, Clock, Forms;

type
	FrameString = string[8]; //in form HHMMSSFF

	TNewFrameEvent = procedure of object;

	TTimeIn = class(TThread)
	private
		{port addresses}
		FBase:WORD;
		BoardFound:boolean;

		FFrame:FrameString; //continuously updated by Execute
			//value is 'HHMMSSFF' format; or '' if timed out
		ClockBox:TClockBox; //form to report the time to, or nil

		FOnNewFrame:TNewFrameEvent;

		//commands to talk to the card
		procedure SendCommand(cmdcode:byte);
		function GetResult:byte;

		procedure NotifyClock;

	public
		property Frame:FrameString read FFrame;
		property OnNewFrame:TNewFrameEvent read FOnNewFrame write FOnNewFrame;

		procedure Reset;
			{call once before use}
		constructor Create(ABase:word; AClockBox:TClockBox);
			{calls reset. ABase should be the card base address, $CC00 is default}

		procedure Execute; override; 		//thread execution
	end;

function AddFrames(fr:FrameString; i:integer; WithDropFrames:boolean):FrameString;
	//add i frames to fr (i can be neg) - optionally handle drop frames
	//in which frames 0-1 of each minute (except each tenth minute) are missing

implementation

const
	halfsec = 1/24/60/60/2;

//DLL functions to call the device driver
function AEC_PCTC_GET_DLL_INFO(lpInfoStr:pointer): integer; stdcall; external 'AEC_NTTC.DLL';
function AEC_PCTC_CHECK_BOARD(lpBrdAddr:Pointer): integer; stdcall; external 'AEC_NTTC.DLL';
function AEC_PCTC_GET_NUMBER_OF_DEVICES: integer; stdcall; external 'AEC_NTTC.DLL';
function AEC_PCTC_SEND_COMMAND(lpCommandB, lpReturnB, lpBrdAddr:Pointer): boolean; stdcall; external 'AEC_NTTC.DLL';
function AEC_PCTC_WRITE_1BYTE(lpWriteB, lpAddrOff, lpBrdAddr:Pointer): boolean; stdcall; external 'AEC_NTTC.DLL';
function AEC_PCTC_READ_1BYTE(lpReadB, lpAddrOff, lpBrdAddr:Pointer): boolean; stdcall; external 'AEC_NTTC.DLL';
function AEC_PCTC_GET_INTCODE(lpWaitMSB, lpReturnB, lpBrdAddr:Pointer): boolean; stdcall; external 'AEC_NTTC.DLL';
function AEC_PCTC_READ_TB(lpTime, lpBrdAddr:Pointer): boolean; stdcall; external 'AEC_NTTC.DLL';
function AEC_PCTC_READ_ASC_TB(lpStrT, lpBrdAddr:Pointer): boolean; stdcall; external 'AEC_NTTC.DLL';

constructor TTimeIn.Create(ABase: word; AClockBox:TClockBox);
var
	i:integer;
begin
	FBase := ABase;
	ClockBox := AClockBox;
	i := AEC_PCTC_CHECK_BOARD(@FBase);
	BoardFound := i = 3;
	Sleep(3000);
	if BoardFound then Reset;
	inherited Create(true); //this sets it to suspended
end;

procedure TTimeIn.SendCommand(cmdcode:byte);
var
	retcode:byte;
	b:boolean;
begin
	b := AEC_PCTC_SEND_COMMAND(@cmdcode, @retcode, @FBase);
	if b then ;
end;

function TTimeIn.GetResult:byte;
var
	wait:byte;
begin
	wait := 10; //ms
	AEC_PCTC_GET_INTCODE(@wait, @result, @FBase);
end;

procedure TTimeIn.Reset;
begin
	SendCommand($21); //LTC reader mode
end;

procedure TTimeIn.Execute;
var
	b:byte;
	s:string[12];
	newframe:FrameString;
begin
	if not BoardFound then exit;
	while not Terminated do begin
		Sleep(5);
		Application.ProcessMessages;
		b := GetResult;
		if b = $13 then begin //$13 = time code available

			//set result
			s := '123456789012';
			AEC_PCTC_READ_ASC_TB(@s[1], @FBase);
			newframe := copy(s, 1, 2) + copy(s, 4, 2) + copy(s, 7, 2) + copy(s, 10, 2);
				//remove colons from s

			//if changed, update clock and call event
			if newframe <> FFrame then begin
				FFrame := newframe;
				if ClockBox <> nil then Synchronize(NotifyClock);
				if Assigned(FOnNewFrame) then Synchronize(FOnNewFrame);
			end;
		end;
	end;
end;

procedure TTimeIn.NotifyClock;
begin
	ClockBox.ShowFrame(FFrame);
end;

function AddFrames(fr:FrameString; i:integer; WithDropFrames:boolean):FrameString;
var
	h,m,s,f:integer; //components of fr
begin
	h := StrToInt(copy(fr, 1, 2));
	m := StrToInt(copy(fr, 3, 2));
	s := StrToInt(copy(fr, 5, 2));
	f := StrToInt(copy(fr, 7, 2));

	//ADDING: take chunks off of i and increment m/s/f until i = 0
	while i > 0 do begin
		if f + i < 30 then begin
			//still within the second:
			inc(f, i);
			i := 0;
		end else begin
			//go to beginning of next second:
			dec(i, 30 - f);
			f := 0;
			inc(s);
			//but if this goes to next minute, fix:
			if s = 60 then begin
				s := 0;
				inc(m);
				if withDropFrames and (m mod 10 <> 0) then f := 2; //skip over 2 drop frames each minute, except on each tenth minute
			end;
		end;
	end;

	//SUBTRACTING: take chunks off of (-i) and decrement m/s/f until i = 0
	while i < 0 do begin

		//if the current second is one with drop frames (ie has only 28 frames)
		if withDropFrames and (s = 0) and (m mod 10 <> 0) then begin
			if (-i) <= f - 2 then begin
				//still within the second:
				inc(f, i);
				i := 0;
			end else begin
				//go to last frame of previous second:
				inc(i, f - 1);
				f := 29;
				dec(s);
				//but if this goes to previous minute, fix:
				if s = -1 then begin
					s := 59;
					dec(m);
				end;
			end;

		//if the current second is a normal one (non drop frame)
		end else begin
			if (-i) <= f then begin
				//still within the second:
				inc(f, i);
				i := 0;
			end else begin
				//go to last frame of previous second:
				inc(i, f + 1);
				f := 29;
				dec(s);
				//but if this goes to previous minute, fix:
				if s = -1 then begin
					s := 59;
					dec(m);
				end;
			end;
		end;
	end;

	//fix invalid minutes
	while m > 59 do begin
		dec(m, 60);
		inc(h);
	end;
	while m < 0 do begin
		inc(m, 60);
		dec(h);
	end;

	//handle negatives or superlarge values
	h := h mod 100;
	while h < 0 do h := h + 100;

	//set result
	result := format('%2.2d%2.2d%2.2d%2.2d', [h, m, s, f]);
end;

end.
